#!/usr/bin/python
#	This script analyse the data acquired by par-logger.ko and
#	saved to files by logdata.
#	Copyright (C) 2005  Kristian Benoit, Opersys inc.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
import getopt
import re
import random

def average(values):
	"""Computes the averages of the list values"""
	return ((sum(values) + 0.0)/len(values))

def stddev(values):
	"""Estimate the standard deviation of the list values"""
	nbval = 4096
	rand = random.Random()
	values = rand.sample(values, nbval)
	sum_square = pow(sum(values), 2)
	square_sum = 0.0
	for val in values:
		square_sum += val*val
	diff = nbval*square_sum - sum_square
	div = diff / (nbval*(nbval-1))
	return (pow(div, 0.5))

def analyse (argv):
	delta = []
	print_delta = False
	print_summary = True
	
	cpuinfo = open("/proc/cpuinfo")
	cpuspeed = cpuinfo.readline()
	
	while re.search("MHz", cpuspeed) == None:
		cpuspeed = cpuinfo.readline()
	
	cpuspeed = float(re.sub('^(\D*)([0-9.]*)(\D*)$',r'\2', cpuspeed))
	divisor = cpuspeed
	unity = " us"
	
	args,filenames = getopt.gnu_getopt(argv[1:],"dascf:")
	
	if len(filenames) == 0:
		filenames = ["data"]
	
	for p in args:
		if p[0] == "-d": # Show only the deltas
			print_summary=False
			print_delta=True
		elif p[0] == "-a": # Show all (the summary and the deltas)
			print_summary=True
			print_delta=True
		elif p[0] == "-s": # Show only the summary
			print_summary=True
			print_delta=False
		if p[0] == "-c": # Show as cycles instead of us
			divisor = 1
			unity = " cycles"
		elif p[0] == "-f": # Set the cpu frequency
			divisor = float(p[1])
			cpuspeed = divisor
			unity = " us"
	
#	print "CPU Freq is : " + str(divisor)
#	print
	
	bad_delta=0
	for filename in filenames:
#		print "READING: " + filename
		datafile = open(filename, 'r')
		line = datafile.readline()
		while line != "":
			val = int(line)
			if val == 0:
				bad_delta += 1
			else:
				delta += [val]
			line = datafile.readline()
	
		datafile.close()
	
	retval={}
	if print_delta:
		retval["deltas"] = delta
	
	if print_summary:
		retval["avr"] = average(delta)/divisor
		retval["max"] = max(delta)/divisor
		retval["min"] = min(delta)/divisor
		retval["stddev"] = stddev(delta)/divisor
#		print
#		print "Number of delta : " + str(len(delta)) + " deltas"
#		print "Number of bad delta : " + str(bad_delta) + " deltas"
#		print "Minimum delta : " + str(min(delta)/divisor) + unity
#		print "Maximum delta : " + str(max(delta)/divisor) + unity
#		print "Average : " + str(average(delta)/divisor) + unity
#		print "Standard deviation : " + str(stddev(delta)/divisor) + unity
#		print
	return retval
